@@ -1,14 +1,14 @@
 
 /*
- * Author: 
+ * Author:
  *      Guido Draheim <guidod@gmx.de>
  *      Tomi Ollila <too@iki.fi>
  *
  * Copyright (c) 1999,2000,2001,2002,2003 Guido Draheim
  *          All rights reserved,
- *          use under the restrictions of the 
+ *          use under the restrictions of the
  *          Lesser GNU General Public License
- *          or alternatively the restrictions 
+ *          or alternatively the restrictions
  *          of the Mozilla Public License 1.1
  */
 
@@ -31,8 +31,10 @@
 
 #define __sizeof(X) ((zzip_ssize_t)(sizeof(X)))
 
+#ifndef ZZIP_EASY
 /* per default, we use a little hack to correct bad z_rootseek parts */
 #define ZZIP_CORRECT_ROOTSEEK 1
+#endif
 
 #if (__GNUC__ == 3 && __GNUC_MINOR__ >= 3) || (__GNUC__ >= 4)
 # ifdef DEBUG
@@ -43,7 +45,7 @@
 #define _255 255
 #endif
 
-#define ZZIP_DISK64_TRAILER
+#define ZZIP_DISK64_TRAILER 1
 
 #ifdef ZZIP_DISK64_TRAILER
 struct _disk_trailer
@@ -117,7 +119,7 @@ __fixup_rootseek(zzip_off_t offset_of_trailer, struct _disk_trailer *trailer)
 
 #elif defined ZZIP_CORRECT_ROOTSEEK
 
-/* store the seekvalue of the trailer into the "z_magic" field and with 
+/* store the seekvalue of the trailer into the "z_magic" field and with
  * a 64bit off_t we overwrite z_disk/z_finaldisk as well. If you change
  * anything in zziplib or dump the trailer structure then watch out that
  * these are still unused, so that this code may still (ab)use those. */
@@ -140,15 +142,15 @@ __debug_dir_hdr(struct zzip_dir_hdr *hdr)
         { WARN1("internal sizeof-mismatch may break wreakage"); }
     /*  the internal directory structure is never bigger than the
      *  external zip central directory space had been beforehand
-     *  (as long as the following assertion holds...) 
+     *  (as long as the following assertion holds...)
      */
 
     if (((zzip_off_t) hdr) & 3)
     {
         NOTE1("this machine's malloc(3) returns sth. not u32-aligned");
     }
-    /* we assume that if this machine's malloc has returned a non-aligned 
-     * memory block, then it is actually safe to access misaligned data, and 
+    /* we assume that if this machine's malloc has returned a non-aligned
+     * memory block, then it is actually safe to access misaligned data, and
      * since it does only affect the first hdr it should not even bring about
      * too much of that cpu's speed penalty
      */
@@ -400,7 +402,7 @@ __zzip_parse_root_directory(int fd,
     struct zzip_dir_hdr *hdr;
     struct zzip_dir_hdr *hdr0;
     uint16_t *p_reclen = 0;
-    unsigned long entries;      /* 32bit is enough */
+    zzip_off64_t entries;
     zzip_off64_t zz_offset;     /* offset from start of root directory */
     char *fd_map = 0;
     zzip_off64_t zz_fd_gap = 0;
@@ -436,11 +438,21 @@ __zzip_parse_root_directory(int fd,
         }
     }
 
-    for (entries = zz_entries, zz_offset = 0; entries; entries--)
+    for (entries=0, zz_offset=0; ; entries++)
     {
         register struct zzip_disk_entry *d;
         uint16_t u_extras, u_comment, u_namlen;
 
+#     ifndef ZZIP_ALLOW_MODULO_ENTRIES
+        if (entries >= zz_entries) {
+            if (zz_offset + 256 < zz_rootsize) {
+                FAIL4("%li's entry is long before the end of directory - enable modulo_entries? (O:%li R:%li)",
+                      (long) entries, (long) (zz_offset), (long) zz_rootsize);
+            }
+            break;
+        }
+#     endif
+
         if (fd_map)
             { d = (void*)(fd_map+zz_fd_gap+zz_offset); } /* fd_map+fd_gap==u_rootseek */
         else
@@ -452,6 +464,14 @@ __zzip_parse_root_directory(int fd,
             d = &dirent;
         }
 
+        if (! zzip_disk_entry_check_magic(d)) {
+#        ifndef ZZIP_ALLOW_MODULO_ENTRIES
+            FAIL4("%li's entry has no disk_entry magic indicator (O:%li R:%li)",
+                  (long) entries, (long) (zz_offset), (long) zz_rootsize);
+#        endif
+            break;
+        }
+
         if ((zzip_off64_t) (zz_offset + sizeof(*d)) > zz_rootsize ||
             (zzip_off64_t) (zz_offset + sizeof(*d)) < 0)
         {
@@ -506,7 +526,7 @@ __zzip_parse_root_directory(int fd,
         {
             FAIL3("%li's entry stretches beyond root directory (O:%li)",
                   (long) entries, (long) (zz_offset));
-            entries--;
+            entries ++;
             break;
         }
 
@@ -540,7 +560,11 @@ __zzip_parse_root_directory(int fd,
         if (hdr_return)
             *hdr_return = hdr0;
     }                           /* else zero (sane) entries */
-    return (entries ? ZZIP_CORRUPTED : 0);
+#  ifndef ZZIP_ALLOW_MODULO_ENTRIES
+    return (entries != zz_entries ? ZZIP_CORRUPTED : 0);
+#  else
+    return ((entries & (unsigned)0xFFFF) != zz_entries ? ZZIP_CORRUPTED : 0);
+#  endif
 }
 
 /* ------------------------- high-level interface ------------------------- */
@@ -557,7 +581,7 @@ zzip_get_default_ext(void)
        ".zip", ".ZIP", /* common extension */
 #     ifdef ZZIP_USE_ZIPLIKES
        ".pk3", ".PK3", /* ID Software's Quake3 zipfiles */
-       ".jar", ".JAR", /* Java zipfiles */ 
+       ".jar", ".JAR", /* Java zipfiles */
 #      endif
 	/* *INDENT-ON* */
         0
@@ -567,7 +591,7 @@ zzip_get_default_ext(void)
 }
 
 /**
- * allocate a new ZZIP_DIR handle and do basic 
+ * allocate a new ZZIP_DIR handle and do basic
  * initializations before usage by => zzip_dir_fdopen
  * => zzip_dir_open => zzip_file_open or through
  * => zzip_open
@@ -600,11 +624,11 @@ zzip_dir_alloc(zzip_strings_t * fileext)
 }
 
 /**
- * will free the zzip_dir handle unless there are still 
+ * will free the zzip_dir handle unless there are still
  * zzip_files attached (that may use its cache buffer).
  * This is the inverse of => zzip_dir_alloc , and both
  * are helper functions used implicitly in other zzipcalls
- * e.g. => zzip_dir_close = zzip_close 
+ * e.g. => zzip_dir_close = zzip_close
  *
  * returns zero on sucess
  * returns the refcount when files are attached.
@@ -630,7 +654,7 @@ zzip_dir_free(ZZIP_DIR * dir)
 }
 
 /** => zzip_dir_free
- * It will also => free(2) the => ZZIP_DIR-handle given. 
+ * It will also => free(2) the => ZZIP_DIR-handle given.
  * the counterpart for => zzip_dir_open
  * see also => zzip_dir_free
  */
@@ -641,14 +665,14 @@ zzip_dir_close(ZZIP_DIR * dir)
     return zzip_dir_free(dir);
 }
 
-/** 
+/**
  * used by the => zzip_dir_open and zzip_opendir(2) call. Opens the
  * zip-archive as specified with the fd which points to an
  * already openend file. This function then search and parse
  * the zip's central directory.
- *  
- * NOTE: refcount is zero, so an _open/_close pair will also delete 
- *       this _dirhandle 
+ *
+ * NOTE: refcount is zero, so an _open/_close pair will also delete
+ *       this _dirhandle
  */
 ZZIP_DIR *
 zzip_dir_fdopen(int fd, zzip_error_t * errcode_p)
@@ -659,7 +683,7 @@ zzip_dir_fdopen(int fd, zzip_error_t * errcode_p)
 static zzip_error_t __zzip_dir_parse(ZZIP_DIR * dir);   /* forward */
 
 /** => zzip_dir_fdopen
- * this function uses explicit ext and io instead of the internal 
+ * this function uses explicit ext and io instead of the internal
  * defaults, setting these to zero is equivalent to => zzip_dir_fdopen
  */
 ZZIP_DIR *
@@ -696,8 +720,8 @@ __zzip_dir_parse(ZZIP_DIR * dir)
     zzip_error_t rv;
     zzip_off_t filesize;
     struct _disk_trailer trailer;
-    /* if (! dir || dir->fd < 0) 
-     *     { rv = EINVAL; goto error; } 
+    /* if (! dir || dir->fd < 0)
+     *     { rv = EINVAL; goto error; }
      */
 
     HINT2("------------------ fd=%i", (int) dir->fd);
@@ -724,7 +748,7 @@ __zzip_dir_parse(ZZIP_DIR * dir)
 
 /**
  * This function will attach any of the .zip extensions then
- * trying to open it the with => open(2). This is a helper 
+ * trying to open it the with => open(2). This is a helper
  * function for => zzip_dir_open, => zzip_opendir and => zzip_open.
  *
  * This function returns a new system file handle or -1 on error.
@@ -760,7 +784,7 @@ __zzip_try_open(zzip_char_t * filename, int filemode,
 
 /**
  * Opens the zip-archive (if available).
- * the two ext_io arguments will default to use posix io and 
+ * the two ext_io arguments will default to use posix io and
  * a set of default fileext that can atleast add .zip ext itself.
  */
 ZZIP_DIR *
@@ -770,7 +794,7 @@ zzip_dir_open(zzip_char_t * filename, zzip_error_t * e)
 }
 
 /** => zzip_dir_open
- * this function uses explicit ext and io instead of the internal 
+ * this function uses explicit ext and io instead of the internal
  * defaults. Setting these to zero is equivalent to => zzip_dir_open
  */
 ZZIP_DIR *
@@ -804,9 +828,9 @@ zzip_dir_open_ext_io(zzip_char_t * filename, zzip_error_t * e,
 }
 
 /** => zzip_dir_open
- * fills the dirent-argument with the values and 
+ * fills the dirent-argument with the values and
  * increments the read-pointer of the dir-argument.
- * 
+ *
  * returns 0 if there no entry (anymore).
  */
 int
@@ -832,7 +856,7 @@ zzip_dir_read(ZZIP_DIR * dir, ZZIP_DIRENT * d)
     return 1;
 }
 
-/* 
+/*
  * Local variables:
  * c-file-style: "stroustrup"
  * End:
